
# q-expansion of newform 425.2.a.g, downloaded from the LMFDB on 14 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 425
weight = 2
poly_data = [5, -2, -6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[-3, -1, 1, 0], 1], [[2, -4, -1, 1], 1]]

hecke_ring_character_values = None
aps_data = [[-1, 0, 0, 1], [-1, 1, 0, 0], [0, 0, 0, 0], [-2, 0, 0, -1], [0, -1, -1, -1], [-1, -2, 1, -1], [1, 0, 0, 0], [0, 2, 0, 2], [-1, -1, -2, 0], [0, 0, 2, -2], [-3, 2, 3, 0], [-4, 0, -2, 2], [-3, 1, 0, 3], [-5, 2, 1, 1], [-1, -2, 1, -1], [-2, 4, 4, 0], [-2, 0, 0, 0], [1, -3, -4, 1], [-1, 0, 3, -1], [-2, 1, 3, -1], [-3, -1, -4, 5], [-1, 0, -1, -4], [5, -4, -1, -3], [6, 3, -1, 0], [2, 2, -2, -2], [4, 3, 5, -2], [-9, -2, -1, 1], [-1, 1, 0, 6], [-1, -1, 0, 1], [-1, 5, 0, -3], [-13, 2, 1, 5], [-5, 2, -3, -2], [-1, -6, -3, 3], [0, 7, -1, -3], [-8, -2, -6, -2], [0, 0, -2, -2], [2, 0, -4, -4], [-10, -4, 0, 7], [-2, 0, 0, -5], [6, -2, -2, 2], [6, -2, 0, 0], [2, -4, 4, -6], [-2, 0, 0, 6], [-1, -1, -2, -3], [-2, 8, 4, 4], [-7, -8, -3, 4], [3, -4, -5, 6], [-5, 2, -5, -1], [0, -6, 0, -5], [-4, -5, 1, -6], [2, 0, 4, -4], [0, 10, 2, 4], [-5, -5, 0, -1], [4, -6, 0, 4], [-1, 2, -3, 3], [-9, -6, -7, -3], [-9, -9, -12, 9], [14, -2, 0, -6], [-5, 1, 4, 1], [-2, 8, -2, 0], [-4, -4, -2, 7], [20, 2, 2, -4], [3, -8, 1, 5], [-6, -7, -1, 3], [1, -9, -4, 1], [19, 3, -2, 3], [4, -2, 0, -14], [2, 6, 10, -8], [-2, 2, -10, 7], [6, 10, 8, -4], [14, 4, -2, 2], [18, -6, 0, -4], [-6, 8, 8, -1], [7, 4, 11, -5], [-21, -4, -5, -2], [-5, -4, 9, -1], [-8, 1, 1, 6], [11, -5, 2, -3], [-8, 14, 8, 4], [-6, 0, -4, 4], [-1, -14, 5, 4], [-2, 3, -3, 4], [1, 0, -7, 12], [5, 12, 1, -3], [-4, -3, 1, -3], [21, -8, -1, -9], [-15, 11, 6, 7], [3, 8, 3, 7], [6, -2, -8, 8], [-21, 6, -1, 5], [15, 8, 3, -7], [-5, 6, 9, 0], [-30, 0, 0, 7], [-20, -4, 0, 10], [11, -2, 9, -8], [-10, 2, 2, 3], [4, 8, -2, 10], [-22, -4, -4, -6], [-5, 16, 7, 3], [-6, 0, -4, -12], [-9, -9, -10, -8], [13, 6, 7, -7], [7, 10, 3, 3], [-4, 10, 4, 6], [-9, -4, -1, 4], [-7, -4, -11, -3], [-9, -2, 11, -9], [-6, 0, -12, -8], [2, -8, 0, 2], [23, -1, 0, -13], [-9, -5, 6, -10], [-6, -10, -8, 10], [-1, -3, -4, -15], [12, -3, -7, 3], [8, -6, 8, -6], [-1, 1, 10, -3], [25, 7, 4, -6], [21, 2, 7, -3], [-14, -10, 2, 2], [14, -4, -10, 2], [-10, 8, -2, 12], [-5, 13, 0, 21], [-11, -9, -12, 7], [-9, 11, -2, 6], [16, -5, 3, -1], [0, -9, -17, 2], [26, -6, -2, -14], [10, -9, 1, -5], [-17, -4, 9, 1], [8, -12, -4, -20], [2, 8, -8, -4], [11, -9, 2, 8], [-36, -9, -9, 21], [5, 2, 3, 1], [20, 1, -1, -6], [6, -1, -1, 12], [5, 10, 11, -11], [18, 10, -6, 7], [-22, 18, 4, 6], [-4, 4, 18, -12], [-2, 23, 5, -3], [17, -1, 6, -17], [-23, 11, 12, -4], [11, 1, 4, 10], [-10, 4, 4, 16], [-4, 7, -1, -3], [30, 2, 6, -8], [-13, -3, 16, -17], [-8, 10, 2, 6], [-13, -4, -9, -9], [-9, 5, 0, -5], [-27, 5, 8, -7], [7, -4, 1, -23], [-1, 13, -8, -8], [8, -14, -8, -9], [-12, -11, -19, 13], [6, -14, 2, -2], [-7, -11, -4, -13], [8, 0, 10, -14], [0, 8, 2, 20], [12, 14, 0, 7], [17, -6, -1, -3], [-9, -6, -15, 5], [22, 4, 10, -12], [2, -2, -6, 4], [-16, -12, 2, 13], [2, -15, 3, 1], [-20, -16, -14, 16]]
